var VarSearch = function()
{
	var varsearch = {};
	var HOME = null;

	varsearch.VERSION = '1.0-beta-5';

	varsearch.checkIsAdmin = function(message, callback, servletDir)
	{
		var url = (servletDir || '../') + 'Session.servlet?ID='+App.getSessionId();
		url += '&cmd=CheckIsAdmin';
		if (message) url += '&message='+encodeURIComponent(message);
		if (!callback) callback = function() {};
		Wizard.asyncJsonRequest(url, callback);
	}

	varsearch.initElements = function(element, autoInit)
	{
		if (autoInit == 'varsearch-options') 
		{
			Events.addEventHandler(element, 'click', varsearch.showOptions);
		}
		else if (autoInit == 'varsearch-hit')
		{
			Events.addEventHandler(element, 'click', varsearch.showHitDetails);
		}
		else if (autoInit == 'varsearch-hide-col')
		{
			var draggable = element.parentNode;
			Events.addEventHandler(element, 'click', varsearch.hideColumn);
			Events.addEventHandler(draggable, 'dragstart', varsearch.beginColumnDrag);
			Events.addEventHandler(draggable, 'dragover', varsearch.checkColumnDropTarget);
			Events.addEventHandler(draggable, 'dragleave', varsearch.leaveColumnDropTarget);
			Events.addEventHandler(draggable, 'drop', varsearch.columnDropped);
			Events.addEventHandler(draggable, 'dragend', varsearch.endColumnDrag);
		}
	}

	varsearch.showOptions = function(event)
	{
		var url = varsearch.getHome()+'options.jsp?ID='+App.getSessionId();
		url += '&indexId='+Data.get(event.currentTarget, 'index-id');
		Dialogs.openPopup(url, 'VarSearchOptions', 600, 400);
	}
	
	varsearch.setVisibleColumns = function(indexId, columns)
	{
		var url = varsearch.getHome()+'Session.servlet?ID='+App.getSessionId();
		url += '&cmd=SetVisibleColumns';
		url += '&indexId='+encodeURIComponent(indexId);
		url += '&visibleColumns='+encodeURIComponent(columns);
		Wizard.asyncJsonRequest(url, varsearch.reloadPage, 'POST');
	}
	
	varsearch.hideColumn = function(event)
	{
		var element = event.currentTarget.parentNode;
		var colToHide = Data.get(element, 'col');
		var indexId = Data.get(element, 'index-id');
		// Find all other columns for the same index
		var visible = varsearch.getVisibleColumn(indexId, colToHide);
		varsearch.setVisibleColumns(indexId, visible.join(','));
	}

	/**
		Get all visible column for the given index.
	*/
	varsearch.getVisibleColumns = function(indexId, except)
	{
		var visible = [];
		var cols = document.getElementsByClassName('hide-col');
		for (var i = 0; i < cols.length; i++)
		{
			var col = cols[i].parentNode;
			if (Data.get(col, 'index-id') == indexId)
			{
				if (except && Data.get(col, 'col') == except) continue;
				visible[visible.length] = Data.get(col, 'col');
			}
		}
		return visible;
	}

	/**
		Start dragging a table column. Store table-id, column-id 
		and column-index on the table element.
	*/
	varsearch.beginColumnDrag = function(event)
	{
		var dragSrc = {};
		dragSrc.indexId = Data.get(event.currentTarget, 'index-id');
		dragSrc.col = Data.get(event.currentTarget, 'col');
		dragSrc.colNo = Data.int(event.currentTarget, 'col-no');
		
		// Store the drag data on the <div> grid element
		Doc.element('varsearch-header.'+dragSrc.indexId).dragSrc = dragSrc;
		event.dataTransfer.effectAllowed = 'move';
		// In FF, we also need to call setData() or the drag event will not start
		// but this results in an exception in IE!!!
		try
		{
			event.dataTransfer.setData('application/json', JSON.stringify(dragSrc));
		}
		catch (e)
		{}
	}

	/**
		Checks if dropping is allowed at this target.
		We only allow dragging within the same table,
		and the target column must be different from the 
		source column.
	*/
	varsearch.checkColumnDropTarget = function(event)
	{
		var indexId = Data.get(event.currentTarget, 'index-id');
		var dragSrc = Doc.element('varsearch-header.'+indexId).dragSrc;
		
		var dropLocation = varsearch.checkTheDropTarget(dragSrc, event.currentTarget);
		if (dropLocation)
		{
			// Enable drop target!
			event.preventDefault();
			event.dataTransfer.dropEffect = 'move';
			// Left or right depending on the order or columns
			Doc.addClass(event.currentTarget, dropLocation < 0 ? 'table-drop-left' : 'table-drop-right');
		}
	}
	
	/**
		Remove drop indicator classes when leaving.
	*/
	varsearch.leaveColumnDropTarget = function(event)
	{
		Doc.removeClass(event.currentTarget, 'table-drop-right');
		Doc.removeClass(event.currentTarget, 'table-drop-left');
	}

	/**
		Another column was dropped here!
	*/
	varsearch.columnDropped = function(event)
	{
		App.debug('columnDropped');
		Doc.removeClass(event.currentTarget, 'table-drop-right');
		Doc.removeClass(event.currentTarget, 'table-drop-left');
		
		// Check that we can drop here
		var indexId = Data.get(event.currentTarget, 'index-id');
		var dragSrc = Doc.element('varsearch-header.'+indexId).dragSrc;
		if (!varsearch.checkTheDropTarget(dragSrc, event.currentTarget)) return;
		
		// Get the existing columns as an array
		var colArray = varsearch.getVisibleColumns(indexId);
		
		var indexSrc = colArray.indexOf(dragSrc.col);
		var indexDest = colArray.indexOf(Data.get(event.currentTarget, 'col'));
		// Remove the dragged column from the array
		if (indexSrc >= 0) colArray.splice(indexSrc, 1);
		// Insert the dragged column before or after the column it was dropped on
		if (indexDest >= 0) colArray.splice(indexDest, 0, dragSrc.col);
		
		// Update and submit the table
		varsearch.setVisibleColumns(indexId, colArray.join(','));
	}


	/**
		Checks if dropping is allowed at this target.
		We only allow dragging within the same table,
		and the target column must be different from the 
		source column.
		returns: 0 = no drop, -1 drop to left, +1 drop to right
	*/
	varsearch.checkTheDropTarget = function(dragSrc, dragTarget)
	{
		if (!dragSrc) return false;
		var indexId = Data.get(dragTarget, 'index-id');
		var colNo = Data.int(dragTarget, 'col-no');
		return indexId == dragSrc.indexId ? colNo - dragSrc.colNo : 0;
	}


	varsearch.showHitDetails = function(event)
	{
		var url = varsearch.getHome()+'hit-details.jsp?ID='+App.getSessionId();
		url += '&rbaId='+Data.get(event.currentTarget, 'rba');
		url += '&fileId='+Data.get(event.currentTarget, 'file');
		url += '&lineNo='+Data.get(event.currentTarget, 'line');
		url += '&idx='+encodeURIComponent(Data.get(event.currentTarget, 'idx'));
		Dialogs.openPopup(url, 'VarSearchHit', 600, 400);
	}

	varsearch.reloadPage = function()
	{
		window.location.reload();
	}

	// Find the <script> tag for 'varsearch.js' and extract the path to it
	varsearch.getHome = function()
	{
		if (HOME != null) return HOME;
		
		var scripts = document.getElementsByTagName('script');
		for (var i = scripts.length - 1; i >= 0; i--)
		{
			var src = scripts[i].src;
			var end = src.indexOf('varsearch.js');
			if (end > 0)
			{
				var start = src.indexOf(App.getRoot());
				HOME = src.substring(start, end);
			}
		}
		// Fallback that should never happen!
		if (HOME == null) HOME = App.getRoot() + 'extensions/varsearch.jar/';
		return HOME;
	}
	
	/**
		Add thousand separators to the count
	*/
	varsearch.niceCount = function(count)
	{
		return count.toString().replace(/(\d)(?=(\d{3})+(?!\d))/g, '$1,');
	}

	return varsearch;
}();

Doc.addElementInitializer(VarSearch.initElements);

var Wizard = function()
{
	var wizard = {};
	var internal = {};
	var debug = 0;
	
	var numLiveAsyncRequests = 0;
	var hasFatalError = false;


	/**
		Submit an ansynchronous request for JSON data. 'GET' method
		is used if 'method' parameter is not set.
		The 'callback' should be a method accepting a single
		JSON parameter object. If the 'status' of the response
		is not 'ok' this method will call setFatalError and
		without calling the callback method.
		
		The 'errorCallback' is an optional function parameter that should
		take two parameters. The first is an Error object and the second is
		the JSON response. The seconds parameter only has a value if the error 
		happens after parsing the response and is null otherwise.
	*/
	wizard.asyncJsonRequest = function(url, callback, method, postdata, errorCallback)
	{
		if (debug)
		{
			App.debug((method || 'GET') + ': ' + url);
			if (postdata) App.debug(postdata);
		}
		var request = Ajax.getXmlHttpRequest();
		request.jsonCallback = callback;
		request.errorCallback = errorCallback;
		request.open(method || 'GET', url, true);
		// Firefox only adds request header with needed "boundary" info, if request header is not set explicitly
		if (postdata && !(postdata instanceof FormData)) 
		{
			request.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
		}
		Ajax.setReadyStateHandler(request, internal.asyncCallbackWrapper, internal.asyncCallbackWrapper);
		numLiveAsyncRequests++;
		request.send(postdata || null);
	}
	
	/**
		Callback wrapper for ansynchronous requests.
	*/
	internal.asyncCallbackWrapper = function(request)
	{
		numLiveAsyncRequests--;
		if (debug) App.debug(request.responseText);
		if (!hasFatalError) Wizard.hideWizardStatus();

		var response = null;
		var messages = null;
		try
		{
			if (request.status != 200)
			{
				throw new Error(request.responseText);
			}
			else
			{
				if (request.getResponseHeader('Content-Type').indexOf('json') != -1)
				{
					response = JSON.parse(request.responseText);
					messages = response.messages;
					if (response.status != 'ok')
					{
						throw new Error(response.message || response.stacktrace);
					}
				}
				else
				{
					response = request.responseText;
				}
				if (request.jsonCallback && request.jsonCallback.call)
				{
					request.jsonCallback.call(null, response);
				}
			}
		}
		catch (err)
		{
			if (request.errorCallback && request.errorCallback.call)
			{
				request.errorCallback.call(null, err, response);
			}
			else
			{
				Wizard.setFatalError(err, messages);
			}
		}
	}
	
	
	/**
		A fatal error has occurred and the wizard will not be able to continue.
		This method will display the error message, disable all form input fields
		and buttons (except the 'Restart' button).
	*/
	wizard.setFatalError = function(err, messages)
	{
		hasFatalError = true;
		var msg = err;
		if (err.toString)
		{
			App.debug(err.toString());
			msg = Strings.encodeTags(err.toString());
			if (err.stack && !messages)
			{
				App.debug(err.stack);
				msg += '<br><pre>'+Strings.encodeTags(err.stack)+'</pre>';
			}
		}
		else
		{
			App.debug(msg);
		}
		
		if (messages) 
		{
			msg = '<div class="messagecontainer error">'+msg+'</div>'+internal.generateMessageList(messages).html;
			wizard.setWizardStatus('success', msg);
		}
		else
		{
			wizard.setWizardStatus('messagecontainer error', msg || 'Unexpected failure!');
		}
		
		Doc.addClass('wizard', 'disabled');
		internal.disableAllFormElements('wizard', true);
		Doc.show('gorestart');
	}
	
	/**
		Display a global wizard-status message. This may for example be
		an error message, a work-in-progress indicator, etc.
	*/
	wizard.setWizardStatus = function(className, msg)
	{
		var status = Doc.element('wizard-status');
		status.className = className;
		status.innerHTML = msg;
		Doc.show(status);
	}
	
	
	/**
		Hide the current global wizard-status message.
	*/
	wizard.hideWizardStatus = function()
	{
		Doc.hide('wizard-status');
		Doc.hide('wizard-progress');
	}
	
		/**
		Display a 'loading' global wizard-status message.
		The 'progress' parameter is optional. If given,
		it should be the name of a progress reporter.
		This activates the display of a progress bar which
		Relax will update at regular intervals in the background.
		See SessionServlet (cmd=GetProgress) for more information.
	*/
	wizard.showLoadingAnimation = function(msg, progress)
	{
		wizard.setWizardStatus('loading', msg  || 'Working, please wait...');
		if (progress)
		{
			var progressBar = Doc.element('wizard-progress');
			progressBar.classList.add('progress-bar');

			// Get/create child elements
			var done = Doc.element('wizard-progress-done');
			if (!done)
			{
				done = document.createElement('span');
				done.id = 'wizard-progress-done';
				done.className = 'progress-bar-done bg-filled-100 topborder bottomborder leftborder';
				progressBar.appendChild(done);
			}
			var remain = Doc.element('wizard-progress-remain');
			if (!remain)
			{
				remain = document.createElement('span');
				remain.id = 'wizard-progress-remain';
				remain.className = 'progress-bar-remain topborder bottomborder rightborder';
				progressBar.appendChild(remain);
			}
				
			progressBar.setAttribute('value', '0%');
			done.style.width = '0%';
			remain.style.width = '100%';
			Doc.show(progressBar);
			
			// Initial request for updating the progress bar
			progressTimer = setTimeout(internal.requestProgressUpdate, 1500, progress);
		}
	}

	/**
		Make a request for the progress information. The 'progress'
		parameter is the name of a progress bar.
	*/
	internal.requestProgressUpdate = function(progress)
	{
		if (hasFatalError) return;
		var url = '../Session.servlet?ID='+App.getSessionId();
		url += '&cmd=GetProgress';
		url += '&name='+encodeURIComponent(progress);
		
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, internal.onProgressReport, internal.onProgressReport);
		request.send(null);
	}
	
	/**
		Callback when receiving a progress update.
	*/
	internal.onProgressReport = function(request)
	{
		if (hasFatalError) return;
		if (debug) App.debug(Strings.encodeTags(request.responseText));
		var response;
		var error = false;
		try
		{
			response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				error = response.message || response.stacktrace || 'Unexpected error';
			}
		}
		catch (ex)
		{
			error = ex;
		}
		
		if (error)
		{
			return;
		}
	
		var progress = response.progress;
		var progressBar = Doc.element('wizard-progress');
	
		if (!progress) 
		{
			// Give up if no progress information
			Doc.hide(progressBar);
			return;
		}
		
		var done = Doc.element('wizard-progress-done');
		var remain = Doc.element('wizard-progress-remain');
	
		progressBar.setAttribute('value', progress.percent+'%');
		done.style.width = progress.percent+'%';
		remain.style.width = (100-progress.percent)+'%';
		
		if (progress.message)
		{
			Doc.element('wizard-status').innerHTML = Strings.encodeTags(progress.message);
		}
		
		if (progress.percent < 100)
		{
			// Continue to request updates until we 100% has been reached
			progressTimer = setTimeout(internal.requestProgressUpdate, 1500, progress.name);
		}
		else
		{
			progressTimer = null;
			Doc.hide(progressBar);
		}
	}

	// Disable all form elements inside the given container
	internal.disableAllFormElements = function(container, hideButtons)
	{
		container = Doc.element(container);
		if (!container) return;
		var input = container.getElementsByTagName('input');
		for (var i = 0; i < input.length; i++)
		{
			input[i].disabled = true;
		}
		var select = container.getElementsByTagName('select');
		for (var i = 0; i < select.length; i++)
		{
			select[i].disabled = true;
		}
		var text = container.getElementsByTagName('textarea');
		for (var i = 0; i < text.length; i++)
		{
			text[i].disabled = true;
		}
		if (hideButtons)
		{
			internal.hideButtons(container);
		}
	}
	
	// Hide all 'basicbuttons':s in the given container
	internal.hideButtons = function(container)
	{
		container = Doc.element(container);
		var buttons = container.getElementsByClassName('basicbutton');
		for (var i = 0; i < buttons.length; i++)
		{
			Doc.hide(buttons[i]);
		}
	}
	
	
	/**
		Initialize the wizard:
		* Display version number
	*/
	internal.initWizard = function()
	{
		var path = document.getElementsByClassName('path');
		if (path && path.length)
		{
			var varsearchVersion = document.createElement('span');
			varsearchVersion.id = 'varsearch-version';
			varsearchVersion.innerHTML = '[Variant Search '+VarSearch.VERSION+']';
			path[0].appendChild(varsearchVersion);
		}
	}
	
	Doc.addFinalizer(internal.initWizard);
	return wizard;
}();

/**
	Holds one or more filter criterias. Use the 'add*' methods
	to add criterias to the filter, and the 'check(item)' method
	to evaluate all criterias against a single item.
*/
var Filter = function()
{
	this.criterias = [];
}

/**
	Adds a text criteria comparing the given item property to
	a static value (f2).
*/
Filter.prototype.addTextCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.eq;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Adds a text criteria checking if the given item property to
	contains a static value (f2).
*/
Filter.prototype.addContainsCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.contains;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Evaluate all criterias. Return true if all pass, false otherwise.
*/
Filter.prototype.check = function(item)
{
	for (var i = 0; i < this.criterias.length; i++)
	{
		if (!this.criterias[i](item)) return false;
	}
	return true;
}


var Comparator = function()
{
	var cmp = {};
	
	/**
		Return a comparator based on the symbol
		which can be either '>' or '<' to return
		Comparator.gt or Comparator.lt. Otherwise
		the given default or Comparator.eq is returned.
	*/
	cmp.get = function(symbol, defaultCmp)
	{
		var c;
		if (symbol == '<')
		{
			c = cmp.lt;
		}
		else if (symbol == '>')
		{
			c = cmp.gt;
		}
		else
		{
			c = defaultCmp || cmp.eq;
		}
		return c;
	}
	
	// Compare if two values are equal
	cmp.eq = function(v1, v2)
	{
		return v1 == v2;
	}
	
	// Compare if two values are different
	cmp.neq = function(v1, v2)
	{
		return v1 != v2;
	}
	
	// Check if v2 is contained in v1
	cmp.contains = function(v1, v2)
	{
		return v1 != null && v2 != null && v1.indexOf(v2) >= 0;
	}
	
	// Compare if two values are appoximately equals (difference is not more than 0.05)
	cmp.approx = function(v1, v2)
	{
		return Math.abs(v2 - v1) < 0.05;
	}
	
	// Compare if v1 starts with the same string as v2
	cmp.startsWith = function(v1, v2)
	{
		return v1 != null && v1.indexOf(v2) == 0;
	}
	
	// Compare if v1 is greater than or equal to v2
	cmp.gt = function(v1, v2)
	{
		return v1 != null && v2 != null && v1 >= v2;
	}
	
	// Compare if v1 is less than or equal to v2
	cmp.lt = function(v1, v2)
	{
		return v1 != null && v2 != null && v1 <= v2;
	}
	
	return cmp;
}();


